@@ -40,9 +40,10 @@ class LinuxFibreChannel(linuxscsi.LinuxSCSI):
 
         This method only works for Fibre Channel targets that implement a
         single WWNN for all ports, so caller should expect us to return either
-        None or an empty list.
+        explicit channel and targets or wild cards if we cannot determine them.
 
-        :returns: List or None
+        :returns: List of lists with [c, t] entries, the channel and target
+        may be '-' wildcards if unable to determine them.
         """
         # We want the target's WWPNs, so we use the initiator_target_map if
         # present for this hba or default to target_wwns if not present.
@@ -74,17 +75,44 @@ class LinuxFibreChannel(linuxscsi.LinuxSCSI):
             LOG.debug('Could not get HBA channel and SCSI target ID, path: '
                       '%(path)s*, reason: %(reason)s', {'path': path,
                                                         'reason': exc})
-            return None
+            return [['-', '-']]
 
     def rescan_hosts(self, hbas, connection_properties):
+        LOG.debug('Rescaning HBAs %(hbas)s with connection properties '
+                  '%(conn_props)s', {'hbas': hbas,
+                                     'conn_props': connection_properties})
         target_lun = connection_properties['target_lun']
+        get_cts = self._get_hba_channel_scsi_target
 
-        for hba in hbas:
-            # Try to get HBA channel and SCSI target to use as filters
-            cts = self._get_hba_channel_scsi_target(hba, connection_properties)
-            # If we couldn't get the channel and target use wildcards
-            if not cts:
-                cts = [('-', '-')]
+        # Use initiator_target_map provided by backend as HBA exclussion map
+        ports = connection_properties.get('initiator_target_map')
+        if ports:
+            hbas = [hba for hba in hbas if hba['port_name'] in ports]
+            LOG.debug('Using initiator target map to exclude HBAs')
+            process = [(hba, get_cts(hba, connection_properties))
+                       for hba in hbas]
+
+        # With no target map we'll check if target implements single WWNN for
+        # all ports, if it does we only use HBAs connected (info was found),
+        # otherwise we are forced to blindly scan all HBAs.
+        else:
+            with_info = []
+            no_info = []
+
+            for hba in hbas:
+                cts = get_cts(hba, connection_properties)
+                found_info = True
+                for hba_channel, target_id in cts:
+                    if hba_channel == '-' or target_id == '-':
+                        found_info = False
+                target_list = with_info if found_info else no_info
+                target_list.append((hba, cts))
+
+            process = with_info or no_info
+            msg = "implements" if with_info else "doesn't implement"
+            LOG.debug('FC target %s single WWNN for all ports.', msg)
+
+        for hba, cts in process:
             for hba_channel, target_id in cts:
                 LOG.debug('Scanning host %(host)s (wwnn: %(wwnn)s, c: '
                           '%(channel)s, t: %(target)s, l: %(lun)s)',
